@@ -3,6 +3,7 @@ Wild Card Match.
 
 A custom implementation of `glob`.
 """
+from __future__ import annotations
 import os
 import sys
 import re
@@ -12,10 +13,7 @@ import bracex
 from . import _wcparse
 from . import _wcmatch
 from . import util
-from typing import (
-    Optional, Iterator, Iterable, List, AnyStr, Union, Generic,
-    Tuple, Pattern, Callable, Any, Set, Sequence, cast
-)
+from typing import Iterator, Iterable, AnyStr, Generic, Pattern, Callable, Any, Sequence, cast
 
 __all__ = (
     "CASE", "IGNORECASE", "RAWCHARS", "DOTGLOB", "DOTMATCH",
@@ -93,12 +91,12 @@ FLAG_MASK = (
 _RE_PATHLIB_DOT_NORM = (
     re.compile(r'(?:((?<=^)|(?<=/))\.(?:/|$))+'),
     re.compile(br'(?:((?<=^)|(?<=/))\.(?:/|$))+')
-)  # type: Tuple[Pattern[str], Pattern[bytes]]
+)  # type: tuple[Pattern[str], Pattern[bytes]]
 
 _RE_WIN_PATHLIB_DOT_NORM = (
     re.compile(r'(?:((?<=^)|(?<=[\\/]))\.(?:[\\/]|$))+'),
     re.compile(br'(?:((?<=^)|(?<=[\\/]))\.(?:[\\/]|$))+')
-)  # type: Tuple[Pattern[str], Pattern[bytes]]
+)  # type: tuple[Pattern[str], Pattern[bytes]]
 
 
 def _flag_transform(flags: int) -> int:
@@ -185,7 +183,7 @@ class _GlobSplit(Generic[AnyStr]):
             self.sep = '/'
         # Once split, Windows file names will never have `\\` in them,
         # so we can use the Unix magic detect
-        self.magic_symbols = _wcparse._get_magic_symbols(pattern, self.unix, self.flags)[0]  # type: Set[AnyStr]
+        self.magic_symbols = _wcparse._get_magic_symbols(pattern, self.unix, self.flags)[0]  # type: set[AnyStr]
 
     def is_magic(self, name: AnyStr) -> bool:
         """Check if name contains magic characters."""
@@ -272,7 +270,7 @@ class _GlobSplit(Generic[AnyStr]):
 
         return success
 
-    def store(self, value: AnyStr, l: List[_GlobPart], dir_only: bool) -> None:
+    def store(self, value: AnyStr, l: list[_GlobPart], dir_only: bool) -> None:
         """Group patterns by literals and potential magic patterns."""
 
         if l and value in (b'', ''):
@@ -281,7 +279,7 @@ class _GlobSplit(Generic[AnyStr]):
         globstar = value in (b'**', '**') and self.globstar
         magic = self.is_magic(value)
         if magic:
-            v = cast(Pattern[AnyStr], _wcparse._compile(value, self.flags))  # type: Union[Pattern[AnyStr], AnyStr]
+            v = cast(Pattern[AnyStr], _wcparse._compile(value, self.flags))  # type: Pattern[AnyStr] | AnyStr
         else:
             v = value
         if globstar and l and l[-1].is_globstar:
@@ -289,7 +287,7 @@ class _GlobSplit(Generic[AnyStr]):
         else:
             l.append(_GlobPart(v, magic, globstar, dir_only, False))
 
-    def split(self) -> List[_GlobPart]:
+    def split(self) -> list[_GlobPart]:
         """Start parsing the pattern."""
 
         split_index = []
@@ -378,12 +376,12 @@ class Glob(Generic[AnyStr]):
 
     def __init__(
         self,
-        pattern: Union[AnyStr, Sequence[AnyStr]],
+        pattern: AnyStr | Sequence[AnyStr],
         flags: int = 0,
-        root_dir: Optional[Union[AnyStr, 'os.PathLike[AnyStr]']] = None,
-        dir_fd: Optional[int] = None,
+        root_dir: AnyStr | os.PathLike[AnyStr] | None = None,
+        dir_fd: int | None = None,
         limit: int = _wcparse.PATTERN_LIMIT,
-        exclude: Optional[Union[AnyStr, Sequence[AnyStr]]] = None
+        exclude: AnyStr | Sequence[AnyStr] | None = None
     ) -> None:
         """Initialize the directory walker object."""
 
@@ -393,10 +391,10 @@ class Glob(Generic[AnyStr]):
         if epats is not None:
             flags = _wcparse.no_negate_flags(flags)
 
-        self.pattern = []  # type: List[List[_GlobPart]]
-        self.npatterns = []  # type: List[Pattern[AnyStr]]
-        self.seen = set()  # type: Set[AnyStr]
-        self.dir_fd = dir_fd if SUPPORT_DIR_FD else None  # type: Optional[int]
+        self.pattern = []  # type: list[list[_GlobPart]]
+        self.npatterns = []  # type: list[Pattern[AnyStr]]
+        self.seen = set()  # type: set[AnyStr]
+        self.dir_fd = dir_fd if SUPPORT_DIR_FD else None  # type: int | None
         self.nounique = bool(flags & NOUNIQUE)  # type: bool
         self.mark = bool(flags & MARK)  # type: bool
         # Only scan for `.` and `..` if it is specifically requested.
@@ -434,11 +432,11 @@ class Glob(Generic[AnyStr]):
         if isinstance(pats[0], bytes):
             ptype = util.BYTES
             self.current = b'.'  # type: AnyStr
-            self.specials = (b'.', b'..')  # type: Tuple[AnyStr, ...]
+            self.specials = (b'.', b'..')  # type: tuple[AnyStr, ...]
             self.empty = b''  # type: AnyStr
             self.stars = b'**'  # type: AnyStr
             self.sep = b'\\' if forcewin else b'/'  # type: AnyStr
-            self.seps = (b'/', self.sep) if forcewin else (self.sep,)  # type: Tuple[AnyStr, ...]
+            self.seps = (b'/', self.sep) if forcewin else (self.sep,)  # type: tuple[AnyStr, ...]
             self.re_pathlib_norm = cast(Pattern[AnyStr], _RE_WIN_PATHLIB_DOT_NORM[ptype])  # type: Pattern[AnyStr]
             self.re_no_dir = cast(Pattern[AnyStr], _wcparse.RE_WIN_NO_DIR[ptype])  # type: Pattern[AnyStr]
         else:
@@ -466,7 +464,7 @@ class Glob(Generic[AnyStr]):
         if epats is not None:
             self._parse_patterns(epats, force_negate=True)
 
-    def _iter_patterns(self, patterns: Sequence[AnyStr], force_negate: bool = False) -> Iterator[Tuple[bool, AnyStr]]:
+    def _iter_patterns(self, patterns: Sequence[AnyStr], force_negate: bool = False) -> Iterator[tuple[bool, AnyStr]]:
         """Iterate expanded patterns."""
 
         seen = set()
@@ -565,16 +563,16 @@ class Glob(Generic[AnyStr]):
 
         return bool(self.npatterns and self._match_excluded(path, is_dir))
 
-    def _match_literal(self, a: AnyStr, b: Optional[AnyStr] = None) -> bool:
+    def _match_literal(self, a: AnyStr, b: AnyStr | None = None) -> bool:
         """Match two names."""
 
         return a.lower() == b if not self.case_sensitive else a == b
 
-    def _get_matcher(self, target: Optional[Union[AnyStr, Pattern[AnyStr]]]) -> Optional[Callable[..., Any]]:
+    def _get_matcher(self, target: AnyStr | Pattern[AnyStr] | None) -> Callable[..., Any] | None:
         """Get deep match."""
 
         if target is None:
-            matcher = None  # type: Optional[Callable[..., Any]]
+            matcher = None  # type: Callable[..., Any] | None
         elif isinstance(target, (str, bytes)):
             # Plain text match
             if not self.case_sensitive:
@@ -607,13 +605,13 @@ class Glob(Generic[AnyStr]):
         else:
             return os.path.join(self.root_dir, path)
 
-    def _iter(self, curdir: Optional[AnyStr], dir_only: bool, deep: bool) -> Iterator[Tuple[AnyStr, bool, bool, bool]]:
+    def _iter(self, curdir: AnyStr | None, dir_only: bool, deep: bool) -> Iterator[tuple[AnyStr, bool, bool, bool]]:
         """Iterate the directory."""
 
         try:
-            fd = None  # type: Optional[int]
+            fd = None  # type: int | None
             if self.is_abs_pattern and curdir:
-                scandir = curdir  # type: Union[AnyStr, int]
+                scandir = curdir  # type: AnyStr | int
             elif self.dir_fd is not None:
                 fd = scandir = os.open(
                     os.path.join(self.root_dir, curdir) if curdir else self.root_dir,
@@ -652,10 +650,10 @@ class Glob(Generic[AnyStr]):
     def _glob_dir(
         self,
         curdir: AnyStr,
-        matcher: Optional[Callable[..., Any]],
+        matcher: Callable[..., Any] | None,
         dir_only: bool = False,
         deep: bool = False
-    ) -> Iterator[Tuple[AnyStr, bool]]:
+    ) -> Iterator[tuple[AnyStr, bool]]:
         """Recursive directory glob."""
 
         files = list(self._iter(curdir, dir_only, deep))
@@ -673,7 +671,7 @@ class Glob(Generic[AnyStr]):
             if deep and not hidden and is_dir and follow:
                 yield from self._glob_dir(path, matcher, dir_only, deep)
 
-    def _glob(self, curdir: AnyStr, part: _GlobPart, rest: List[_GlobPart]) -> Iterator[Tuple[AnyStr, bool]]:
+    def _glob(self, curdir: AnyStr, part: _GlobPart, rest: list[_GlobPart]) -> Iterator[tuple[AnyStr, bool]]:
         """
         Handle glob flow.
 
@@ -748,7 +746,7 @@ class Glob(Generic[AnyStr]):
                 else:
                     yield path, is_dir
 
-    def _get_starting_paths(self, curdir: AnyStr, dir_only: bool) -> List[Tuple[AnyStr, bool]]:
+    def _get_starting_paths(self, curdir: AnyStr, dir_only: bool) -> list[tuple[AnyStr, bool]]:
         """
         Get the starting location.
 
@@ -797,9 +795,9 @@ class Glob(Generic[AnyStr]):
     def glob(self) -> Iterator[AnyStr]:
         """Starts off the glob iterator."""
 
-        curdir = self.current
-
         for pattern in self.pattern:
+            curdir = self.current
+
             # If the pattern ends with `/` we return the files ending with `/`.
             dir_only = pattern[-1].dir_only if pattern else False
             self.is_abs_pattern = pattern[0].is_drive if pattern else False
@@ -848,13 +846,13 @@ class Glob(Generic[AnyStr]):
 
 
 def iglob(
-    patterns: Union[AnyStr, Sequence[AnyStr]],
+    patterns: AnyStr | Sequence[AnyStr],
     *,
     flags: int = 0,
-    root_dir: Optional[Union[AnyStr, 'os.PathLike[AnyStr]']] = None,
-    dir_fd: Optional[int] = None,
+    root_dir: AnyStr | os.PathLike[AnyStr] | None = None,
+    dir_fd: int | None = None,
     limit: int = _wcparse.PATTERN_LIMIT,
-    exclude: Optional[Union[AnyStr, Sequence[AnyStr]]] = None
+    exclude: AnyStr | Sequence[AnyStr] | None = None
 ) -> Iterator[AnyStr]:
     """Glob."""
 
@@ -865,26 +863,26 @@ def iglob(
 
 
 def glob(
-    patterns: Union[AnyStr, Sequence[AnyStr]],
+    patterns: AnyStr | Sequence[AnyStr],
     *,
     flags: int = 0,
-    root_dir: Optional[Union[AnyStr, 'os.PathLike[AnyStr]']] = None,
-    dir_fd: Optional[int] = None,
+    root_dir: AnyStr | os.PathLike[AnyStr] | None = None,
+    dir_fd: int | None = None,
     limit: int = _wcparse.PATTERN_LIMIT,
-    exclude: Optional[Union[AnyStr, Sequence[AnyStr]]] = None
-) -> List[AnyStr]:
+    exclude: AnyStr | Sequence[AnyStr] | None = None
+) -> list[AnyStr]:
     """Glob."""
 
     return list(iglob(patterns, flags=flags, root_dir=root_dir, dir_fd=dir_fd, limit=limit, exclude=exclude))
 
 
 def translate(
-    patterns: Union[AnyStr, Sequence[AnyStr]],
+    patterns: AnyStr | Sequence[AnyStr],
     *,
     flags: int = 0,
     limit: int = _wcparse.PATTERN_LIMIT,
-    exclude: Optional[Union[AnyStr, Sequence[AnyStr]]] = None
-) -> Tuple[List[AnyStr], List[AnyStr]]:
+    exclude: AnyStr | Sequence[AnyStr] | None = None
+) -> tuple[list[AnyStr], list[AnyStr]]:
     """Translate glob pattern."""
 
     flags = _flag_transform(flags)
@@ -892,14 +890,14 @@ def translate(
 
 
 def globmatch(
-    filename: Union[AnyStr, 'os.PathLike[AnyStr]'],
-    patterns: Union[AnyStr, Sequence[AnyStr]],
+    filename: AnyStr | os.PathLike[AnyStr],
+    patterns: AnyStr | Sequence[AnyStr],
     *,
     flags: int = 0,
-    root_dir: Optional[Union[AnyStr, 'os.PathLike[AnyStr]']] = None,
-    dir_fd: Optional[int] = None,
+    root_dir: AnyStr | os.PathLike[AnyStr] | None = None,
+    dir_fd: int | None = None,
     limit: int = _wcparse.PATTERN_LIMIT,
-    exclude: Optional[Union[AnyStr, Sequence[AnyStr]]] = None
+    exclude: AnyStr | Sequence[AnyStr] | None = None
 ) -> bool:
     """
     Check if filename matches pattern.
@@ -920,15 +918,15 @@ def globmatch(
 
 
 def globfilter(
-    filenames: Iterable[Union[AnyStr, 'os.PathLike[AnyStr]']],
-    patterns: Union[AnyStr, Sequence[AnyStr]],
+    filenames: Iterable[AnyStr | os.PathLike[AnyStr]],
+    patterns: AnyStr | Sequence[AnyStr],
     *,
     flags: int = 0,
-    root_dir: Optional[Union[AnyStr, 'os.PathLike[AnyStr]']] = None,
-    dir_fd: Optional[int] = None,
+    root_dir: AnyStr | os.PathLike[AnyStr] | None = None,
+    dir_fd: int | None = None,
     limit: int = _wcparse.PATTERN_LIMIT,
-    exclude: Optional[Union[AnyStr, Sequence[AnyStr]]] = None
-) -> List[Union[AnyStr, 'os.PathLike[AnyStr]']]:
+    exclude: AnyStr | Sequence[AnyStr] | None = None
+) -> list[AnyStr | os.PathLike[AnyStr]]:
     """Filter names using pattern."""
 
     # Shortcut out if we have no patterns
@@ -937,7 +935,7 @@ def globfilter(
 
     rdir = os.fspath(root_dir) if root_dir is not None else root_dir
 
-    matches = []  # type: List[Union[AnyStr, 'os.PathLike[AnyStr]']]
+    matches = []  # type: list[AnyStr | os.PathLike[AnyStr]]
     flags = _flag_transform(flags)
     obj = _wcparse.compile(patterns, flags, limit, exclude)
 
@@ -949,7 +947,7 @@ def globfilter(
 
 
 @util.deprecated("This function will be removed in 9.0.")
-def raw_escape(pattern: AnyStr, unix: Optional[bool] = None, raw_chars: bool = True) -> AnyStr:
+def raw_escape(pattern: AnyStr, unix: bool | None = None, raw_chars: bool = True) -> AnyStr:
     """Apply raw character transform before applying escape."""
 
     return _wcparse.escape(
@@ -957,7 +955,7 @@ def raw_escape(pattern: AnyStr, unix: Optional[bool] = None, raw_chars: bool = T
     )
 
 
-def escape(pattern: AnyStr, unix: Optional[bool] = None) -> AnyStr:
+def escape(pattern: AnyStr, unix: bool | None = None) -> AnyStr:
     """Escape."""
 
     return _wcparse.escape(pattern, unix=unix)
